@@ -5,8 +5,8 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2022
-;; Package-Version: 20260104.1114
-;; Package-Revision: 8873ab82ec9a
+;; Package-Version: 20260109.1817
+;; Package-Revision: 7880b450138b
 ;; Package-Requires: ((emacs "29.1") (compat "30"))
 ;; URL: https://github.com/minad/osm
 ;; Keywords: network, multimedia, hypermedia, mouse
@@ -36,7 +36,7 @@
 ;; locations using regular Emacs bookmarks or create links from Org files
 ;; to locations.  Furthermore the package provides commands to measure
 ;; distances, search for locations and routes by name and to open and
-;; display GPX tracks.
+;; display GPX or TCX tracks.
 
 ;; osm.el requires Emacs 29 and depends on the external `curl' program.
 ;; Emacs must be built with libxml, libjansson, librsvg, libjpeg, libpng
@@ -600,8 +600,8 @@ Local per buffer since the overlays depend on the zoom level.")
       (when (equal status "200")
         (ignore-errors (rename-file file (string-remove-suffix ".tmp" file) t))
         (osm--each
-          (when (and (= osm--zoom zoom) (eq osm-server server))
-            (osm--display-tile x y (osm--get-tile x y)))))
+         (when (and (= osm--zoom zoom) (eq osm-server server))
+           (osm--display-tile x y (osm--get-tile x y)))))
       (cl-callf2 delete (list server zoom x y) osm--download-active)
       (delete-file file)))
   output)
@@ -623,8 +623,8 @@ Local per buffer since the overlays depend on the zoom level.")
         (push job osm--download-active)
         (cl-incf count)))
     (osm--each
-      (dolist (job jobs)
-        (cl-callf2 delq job osm--download-queue)))
+     (dolist (job jobs)
+       (cl-callf2 delq job osm--download-queue)))
     (cons `("curl" "--disable" "--write-out" "%{http_code} %{filename_effective}\n"
             ,@(split-string-and-unquote osm-curl-options) ,@(nreverse args))
           jobs)))
@@ -658,8 +658,8 @@ Local per buffer since the overlays depend on the zoom level.")
           (force-mode-line-update t)
           (osm--download)))
        (alist-get server osm--download-processes))
-       (force-mode-line-update t)
-       (osm--download))))
+      (force-mode-line-update t)
+      (osm--download))))
 
 (defun osm-mouse-drag (event)
   "Handle drag EVENT."
@@ -969,13 +969,13 @@ Local per buffer since the overlays depend on the zoom level.")
            (pcase-let ((`(,lat ,lon ,zoom) (bookmark-prop-get bm 'coordinates)))
              (funcall fun 'osm-bookmark lat lon zoom (car bm))))
   (cl-loop for (dname id segs waypoints) in osm--datasets do
-    (when-let* ((start (caar segs)))
-      (funcall fun id (car start) (cdr start) 10
-               (propertize dname 'osm-dataset dname)))
-    (cl-loop for (lat lon name) in waypoints do
-             (funcall fun id lat lon 15
-                      (propertize (format "%s [%s]" name dname)
-                                  'osm-dataset dname))))
+           (when-let* ((start (caar segs)))
+             (funcall fun id (car start) (cdr start) 10
+                      (propertize dname 'osm-dataset dname)))
+           (cl-loop for (lat lon name) in waypoints do
+                    (funcall fun id lat lon 15
+                             (propertize (format "%s [%s]" name dname)
+                                         'osm-dataset dname))))
   (cl-loop for (lat lon name) in osm--track do
            (funcall fun 'osm-track lat lon 15 name)))
 
@@ -1189,8 +1189,8 @@ xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink'>
   (clear-image-cache t) ;; Make absolutely sure that the tiles are redrawn.
   (setq osm--tile-cache nil)
   (osm--each
-    (setq osm--overlays nil)
-    (osm--update)))
+   (setq osm--overlays nil)
+   (osm--update)))
 
 (defun osm--header-button (text action)
   "Format header line button with TEXT and ACTION."
@@ -1805,37 +1805,82 @@ See `osm-search-server' and `osm-search-language' for customization."
 
 ;;;###autoload
 (defun osm-open (file)
-  "Show the tracks of GPX FILE in an `osm-mode' buffer."
-  (interactive "fGPX file: ")
+  "Show the tracks of GPX or TCX FILE in an `osm-mode' buffer."
+  (interactive "fGPX or TCX file: ")
   (osm--check-libraries)
   (let ((dom (with-temp-buffer
                (insert-file-contents file)
-               (libxml-parse-xml-region (point-min) (point-max)))))
-    (unless (eq 'gpx (dom-tag dom))
-      (setq dom (dom-child-by-tag dom 'gpx)))
-    (unless (and dom (eq 'gpx (dom-tag dom)))
-      (error "Not a GPX file"))
-    (osm--add-dataset
-     (abbreviate-file-name file)
-     'osm-file
+               (libxml-parse-xml-region (point-min) (point-max))))
+        (file (abbreviate-file-name file)))
+    (if (eq 'TrainingCenterDatabase (dom-tag dom))
+        (osm--add-dataset file 'osm-file
+                          (osm--tcx-track dom)
+                          (osm--tcx-waypoints dom))
+      (unless (eq 'gpx (dom-tag dom))
+        (setq dom (dom-child-by-tag dom 'gpx)))
+      (unless (and dom (eq 'gpx (dom-tag dom)))
+        (error "Not a GPX or TCX file"))
+      (osm--add-dataset file 'osm-file
+                        (osm--gpx-track dom)
+                        (osm--gpx-waypoints dom)))))
+
+;; Prevent deprecation warning
+(defalias 'osm--dom-text 'dom-text)
+
+(defsubst osm--tcx-position (pos)
+  "Convert POS to a cons of (lat . lon)."
+  (let ((lat (dom-child-by-tag pos 'LatitudeDegrees))
+        (lon (dom-child-by-tag pos 'LongitudeDegrees)))
+    (cons (string-to-number (osm--dom-text lat))
+          (string-to-number (osm--dom-text lon)))))
+
+(defun osm--tcx-waypoints (dom)
+  "Return waypoints contained in tcx DOM."
+  (cl-loop
+   with courses = (dom-child-by-tag dom 'Courses)
+   for course in (dom-by-tag courses 'Course) nconc
+   (cl-loop
+    for pt in (dom-by-tag course 'CoursePoint)
+    for pos = (osm--tcx-position (dom-child-by-tag pt 'Position))
+    for name = (dom-by-tag pt 'Name)
+    collect (list (car pos) (cdr pos) (osm--dom-text name)))))
+
+(defun osm--tcx-track (dom)
+  "Return track points contained in tcx DOM."
+  (cl-loop
+   with activities = (dom-child-by-tag dom 'Activities)
+   for activity in (dom-by-tag activities 'Activity) nconc
+   (cl-loop
+    for lap in (dom-by-tag activity 'Lap) nconc
+    (cl-loop
+     for track in (dom-by-tag lap 'Track) collect
      (cl-loop
-      for trk in (dom-children dom)
-      if (eq (dom-tag trk) 'trk) nconc
-      (cl-loop
-       for seg in (dom-children trk)
-       if (eq (dom-tag seg) 'trkseg) collect
-       (cl-loop
-        for pt in (dom-children seg)
-        if (eq (dom-tag pt) 'trkpt) collect
-        (cons (string-to-number (dom-attr pt 'lat))
-              (string-to-number (dom-attr pt 'lon))))))
-     (cl-loop
-      for pt in (dom-children dom)
-      if (eq (dom-tag pt) 'wpt) collect
-      (list (string-to-number (dom-attr pt 'lat))
-            (string-to-number (dom-attr pt 'lon))
-            (with-no-warnings
-              (dom-text (dom-child-by-tag pt 'name))))))))
+      for pt in (dom-by-tag track 'Trackpoint)
+      for pos = (dom-child-by-tag pt 'Position)
+      if pos collect (osm--tcx-position pos))))))
+
+(defun osm--gpx-track (dom)
+  "Return track points contained in gpx DOM."
+  (cl-loop
+   for trk in (dom-children dom)
+   if (eq (dom-tag trk) 'trk) nconc
+   (cl-loop
+    for seg in (dom-children trk)
+    if (eq (dom-tag seg) 'trkseg) collect
+    (cl-loop
+     for pt in (dom-children seg)
+     if (eq (dom-tag pt) 'trkpt) collect
+     (cons (string-to-number (dom-attr pt 'lat))
+           (string-to-number (dom-attr pt 'lon)))))))
+
+(defun osm--gpx-waypoints (dom)
+  "Return waypoints contained in gpx DOM."
+  (cl-loop
+   for pt in (dom-children dom)
+   if (eq (dom-tag pt) 'wpt) collect
+   (list (string-to-number (dom-attr pt 'lat))
+         (string-to-number (dom-attr pt 'lon))
+         (osm--dom-text (dom-child-by-tag pt 'name)))))
 
 (defun osm--add-dataset (name id track waypoints)
   "Add dataset with NAME and ID consisting of TRACK and WAYPOINTS."
